% Calcultes optimal retirement times

% Input: 
% (structural) theta at unnatural parametrization, including tau for
% copula 
% covariate matrices (N*(K-3)/2)
% error draw vectors (N*NREPS * 1)
% rho, f, tau1, tau2
% 
% Output:
% matrix of retirement times (N*NREPS * 2)

% Notes:
% use the domain space stopping criterion: stop when bisection endpoints
% are very close
% inputs here are vectors and matrices, do everything all at once to
% minimize looping
% theta is (ln(delta-1), ln(alpha1), beta1, ln(alpha2), beta2, ln(tau)),
% unnatural parametrization
% steps:
% calculate optimal times assuming t1 < t2; assuming t2 < t1 and assuming
% t1 = t2
% then compare the utility levels
%
% 1 and 2 are the players. a,b,c the cases
% 
% In this version the tau's can have two colums. One for each
% jump-location. They must be ordered;

function [tt] = rettimesT2Jumps(theta, x1, x2, k1, k2, tau1, tau2, rho, f)

% get theta 1 and 2
th1 = theta(1 : (length(theta)/2));
th2 = transpose([theta(1), transpose(theta(((length(theta) / 2) ...
                             + 1) : (length(theta) - 1)))]);

% ensure that the parameters meet the necessary conditions
th2(1) = exp(theta(1)) + 1;
th1(1) = th2(1);
th1(2) = exp(th1(2));
th2(2) = exp(th2(2));
th1(3) = exp(th1(3));
th2(3) = exp(th2(3));
th1(4) = exp(th1(4));
th2(4) = exp(th2(4));

fi1=phi(x1, th1(5:length(th1)));
fi2=phi(x2, th2(5:length(th2)));


%%%% assuming t1 < t2 %%%
% calculate t1 which is given
t1a = ZinvT2Jumps(k1 ./ fi1, th1(2), th1(3:4), tau1);
% lower bound of possible optimun t2 value
l2a = t1a;  
% upper bound of possible optimum t2 value
u2a = max([(ZinvT2Jumps(k2 ./ (fi2 .* th2(1)), th2(2), th2(3:4), tau2)), l2a], [], 2);
% combine lower and upper bounds
b2a = [l2a, u2a];

%%% assuming t2 < t1 %%%
% calculate t2 which is given
t2b = ZinvT2Jumps(k2 ./ fi2, th2(2), th2(3:4), tau2);
% lower bound of possible optimum t1 value
l1b = t2b;  
% upper bound of possible optimum t1 value
u1b = max([(ZinvT2Jumps(k1 ./ (fi1 .* th1(1)), th1(2), th1(3:4), tau1)), l1b], [], 2);
% combine lower and upper bounds
b1b = [l1b, u1b];

%%% assuming t1 = t2 %%%
% calculate lower and upper bounds of possible optimum t value
c1c = ZinvT2Jumps(k1 ./ (fi1 .* th1(1)), th1(2), th1(3:4), tau1);
c2c = ZinvT2Jumps(k2 ./ (fi2 .* th2(1)), th2(2), th2(3:4), tau2);
lc = min([c1c, c2c], [], 2);
uc = max([c1c, c2c], [], 2);
% combine lower and upper bounds
bc = [lc, uc];


%%% call loop function to calculate optimum for each one of the three cases
[t2a, t1b, tc] = loopT2Jumps(b2a, b1b, bc, t1a, t2b, k1, k2, fi1, fi2, th1, th2, tau1, tau2, rho, f);



%%% calculate utility in each one of the three cases

% t1 < t2
ztt = ZtildeT2Jumps(th1(2), t1a, rho, tau1, th1(3:4));
zts = ZtildeT2Jumps(th1(2), t2a, rho, tau1, th1(3:4));
ztt2 = ZtildeT2Jumps(th2(2), t2a, rho, tau2, th2(3:4));
ert1 = exp(-rho .* t1a);
ert2 = exp(-rho .* t2a);
n_a = (Uts(th1, fi1, k1, rho, ztt, zts, ert1) - AT(th1, fi1, k1, f, rho, tau1)) ... 
          .* (Ut(th2, fi2, k2, rho, ztt2, ert2) - AT(th2, fi2, k2, f, rho, tau2));

% t2 < t1
ztt2 = ZtildeT2Jumps(th1(2), t1b, rho, tau1, th1(3:4));
ztt = ZtildeT2Jumps(th2(2), t2b, rho, tau2, th2(3:4));
zts = ZtildeT2Jumps(th2(2), t1b, rho, tau2, th2(3:4));
ert1 = exp(-rho .* t1b);
ert2 = exp(-rho .* t2b);
n_b = (Ut(th1, fi1, k1, rho, ztt2, ert1) - AT(th1, fi1, k1, f, rho, tau1)) ...
          .* (Uts(th2, fi2, k2, rho, ztt, zts, ert2) - AT(th2, fi2, k2, f, rho, tau2));

% t1 = t2
ztt1 = ZtildeT2Jumps(th1(2), tc, rho, tau1, th1(3:4));
ztt2 = ZtildeT2Jumps(th2(2), tc, rho, tau2, th2(3:4));
ertc = exp(-rho .* tc);
n_c = (Ut(th1, fi1, k1, rho, ztt1, ertc) - AT(th1, fi1, k1, f, rho, tau1)) ...
          .* (Ut(th2, fi2, k2, rho, ztt2, ertc) - AT(th2, fi2, k2, f, rho, tau2));

% find out which of the three utilities is the greatest
no_a = (n_a > n_b) .* (n_a > n_c);
no_b = (n_b > n_a) .* (n_b > n_c);
no_c = 1 - no_a - no_b;




% output the corresponding retirement times
tt = bsxfun(@times, no_a, [t1a, t2a]) + ...
     bsxfun(@times, no_b, [t1b, t2b]) + ...
     bsxfun(@times, no_c, [tc, tc]);
end

