% weibul.m - performs MLE on Weibul likelihood function
%
% Other m-files required: none
% Subfunctions: weibul_mle, weibul_mle_obj_foc, 
%   weibul_mle_foc, weibul_mle_soc
%
% C-files required: N/A
%
% Execute:        weibul
%
% Author: Andrew Semelhago
%
% email: andrewis@princeton.edu
%
% February 2015; Last revision: 2015-02-16
%
%------------- BEGIN CODE --------------

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Test Client
function weibul
%b = [1; .5; 2.5; 1];
%x=[2 4 3; 1 5 7; 6 1 9];
%u=[.1231; .7412; .2318];
%U=-log(u);
%T=U.*exp(-x*(b(2:length(b))));
%c=2e-7;
%ceni=c<T;
b = [1; .5; 2.5; 1; 4; .2];
x=randn(10000,5);
u=rand(size(x, 1), 1);
U=-log(u);
T=U.*exp(-x*(b(2:length(b))));
c=4e4;
ceni=c<T;
%weibul_mle_obj_foc(b,T,ceni,x)

%weibul_mle_soc(b,T,ceni,x)

weibul_mle(T, ceni, x)
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%Calculates parameters that maximize Weibul likelihood function%%

%{
Param:
ti: n x 1 vector of times
ceni: n x 1 vector of indicators (true if censoring occured)
xi: n x k matrix of data
%}

%{
Returns:
k x 1 vector of coefficient estimates
%}

function p = weibul_mle(ti, ceni, xi)
bet = -(transpose(xi) * xi)^-1 * (transpose(xi) * log(ti));
alp = 0.3;
p = cat(1, alp, bet);

b = p;
converged = 0;

while not(converged)
    options = optimoptions(@fminunc,'Algorithm', 'quasi-newton', ...
        'GradObj', 'on', 'Display', 'off');

    f = @(beta)weibul_mle_obj_foc(beta, ti, ceni, xi);
    [p, fmax] = fminunc(f, b, options);
    fmax = -fmax;
    converged = 1;
    
    b = p;
    h = 0.001;
    
    for i = 1:length(p)
       pp = p;
       pp(i) = pp(i) + h;
       f1 = weibul_mle_obj_foc(pp, ti, ceni, xi);
       %fprintf('1: f1: %d;  fmax: %d\n', -f1, fmax);
       if -f1 > fmax
           fmax = -f1;
           b = pp;
           converged = 0;
       end
       
       pp = p;
       pp(i) = pp(i) - h;
       f1 = weibul_mle_obj_foc(pp, ti, ceni, xi);
       %fprintf('2: f1: %d;  fmax: %d\n', -f1, fmax);
       if -f1 > fmax
           fmax = -f1;
           b = pp;
           converged = 0;
       end
    end
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%Calculates objective requiring maximization and FOC%%

%{
Param:
beta: k x 1 vector of estimated coefficients
ti: n x 1 vector of times
ceni: n x 1 vector of indicators (true if censoring occured)
xi: n x k matrix of data
%}

%{
Returns:
Objective function and its FOC
%}

function [ll, der] = weibul_mle_obj_foc(beta, ti, ceni, xi)
if (isvector(xi))
    n = length(xi);
else
    n = size(xi, 1);
end

%%% Start of Objective Calculation %%%
bet = beta(2:length(beta));
exb = exp(xi * bet);
tia = ti .^ exp(beta(1));
lti = log(ti);
alpha = exp(beta(1));

ll = ( ...
    sum((1 - ceni) .* (log(alpha) + (alpha - 1) .* log(ti) + ...
        xi * bet)) - sum(tia .* exb) ...
            );

% to maximise objective
ll = -ll / n;

%%% Start of FOC Calculation %%%

dL_dg = (sum((1 - ceni) .* (1 / alpha + lti)) ...
            - sum((tia) .* lti .* exb)) * alpha;
        
dL_db = ones(1, n) * (repmat(1 - ceni, 1, size(xi, 2)) .* xi) ...
                - ones(1, n) * (repmat(tia .* exb, 1, size(xi, 2)) .* xi);

der = - [dL_dg dL_db] / n;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%Calculates SOC of objective%%

%{
Param:
beta: k x 1 vector of estimated coefficients
ti: n x 1 vector of times
ceni: n x 1 vector of indicators (true if censoring occured)
xi: n x k matrix of data
%}

%{
Returns:
SOC of objective
%}

function der = weibul_mle_soc(beta, ti, ceni, xi)
bet = beta(2:length(beta));

d2L_dg2 = (-sum((1 - ceni) .* (1 / exp(beta(1)) .^ 2)) - ...
            sum((ti .^ exp(beta(1))) .* log(ti) .^ 2 .* exp(xi * bet))) * ...
                exp(beta(1)) .^ 2 ...
           - (sum((1 - ceni) .* (1 / exp(beta(1)) + log(ti))) - ...
            sum((ti .^ (exp(beta(1)))) .* log(ti) .* exp(xi * bet))) * ...
                exp(beta(1));

d2L_dgdb = sum(-exp(beta(1)) * ...
            bsxfun(@times, (ti.^exp(beta(1))).*log(ti).*exp(xi*bet), xi), 1);

d2L_db2 = transpose(bsxfun(@times, ti .^ exp(beta(1)) .* exp(xi * bet), xi)) * xi;

der(1, 1) = d2L_dg2;
der(1, 2:length(beta)) = d2L_dgdb;
der(2:length(beta), 1) = d2L_dgdb;
der(2:length(beta), 2:length(beta)) = d2L_db2;

der = der / size(xi, 1);
end
